package org.eclipse.jdt.internal.compiler.parser.diagnose;

package .tmp;

import org.eclipse.jdt.internal.compiler.ast.AbstractMethodDeclaration;
import org.eclipse.jdt.internal.compiler.ast.FieldDeclaration;
import org.eclipse.jdt.internal.compiler.ast.Initializer;
import org.eclipse.jdt.internal.compiler.ast.TypeDeclaration;

public class RangeUtil {
  public static final int NO_FLAG = 0;
  
  public static final int LBRACE_MISSING = 1;
  
  public static final int IGNORE = 2;
  
  public static boolean containsErrorInSignature(AbstractMethodDeclaration method) {
    return !(method.sourceEnd + 1 != method.bodyStart && method.bodyEnd != method.declarationSourceEnd);
  }
  
  public static int[][] computeDietRange(TypeDeclaration[] types) {
    if (types == null || types.length == 0)
      return new int[3][0]; 
    RangeResult result = new RangeResult();
    computeDietRange0(types, result);
    return result.getRanges();
  }
  
  private static void computeDietRange0(TypeDeclaration[] types, RangeResult result) {
    for (int j = 0; j < types.length; j++) {
      TypeDeclaration[] memberTypeDeclarations = (types[j]).memberTypes;
      if (memberTypeDeclarations != null && memberTypeDeclarations.length > 0)
        computeDietRange0((types[j]).memberTypes, result); 
      AbstractMethodDeclaration[] methods = (types[j]).methods;
      if (methods != null) {
        int length = methods.length;
        for (int i = 0; i < length; i++) {
          AbstractMethodDeclaration method = methods[i];
          if (containsIgnoredBody(method))
            if (containsErrorInSignature(method)) {
              method.bits |= 0x20;
              result.addInterval(method.declarationSourceStart, method.declarationSourceEnd, 2);
            } else {
              int flags = (method.sourceEnd + 1 == method.bodyStart) ? 1 : 0;
              result.addInterval(method.bodyStart, method.bodyEnd, flags);
            }  
        } 
      } 
      FieldDeclaration[] fields = (types[j]).fields;
      if (fields != null) {
        int length = fields.length;
        for (int i = 0; i < length; i++) {
          if (fields[i] instanceof Initializer) {
            Initializer initializer = (Initializer)fields[i];
            if (initializer.declarationSourceEnd == initializer.bodyEnd && initializer.declarationSourceStart != initializer.declarationSourceEnd) {
              initializer.bits |= 0x20;
              result.addInterval(initializer.declarationSourceStart, initializer.declarationSourceEnd, 2);
            } else {
              result.addInterval(initializer.bodyStart, initializer.bodyEnd);
            } 
          } 
        } 
      } 
    } 
  }
  
  public static boolean containsIgnoredBody(AbstractMethodDeclaration method) {
    return (!method.isDefaultConstructor() && 
      !method.isClinit() && (
      method.modifiers & 0x1000000) == 0);
  }
}
